@@ -108,14 +108,18 @@ class GCSClient(luigi.target.FileSystem):
       as the ``descriptor`` argument.
     """
     def __init__(self, oauth_credentials=None, descriptor='', http_=None,
-                 chunksize=CHUNKSIZE):
+                 chunksize=CHUNKSIZE, **discovery_build_kwargs):
         self.chunksize = chunksize
         authenticate_kwargs = gcp.get_authenticate_kwargs(oauth_credentials, http_)
 
+        build_kwargs = authenticate_kwargs.copy()
+        build_kwargs.update(discovery_build_kwargs)
+
         if descriptor:
-            self.client = discovery.build_from_document(descriptor, **authenticate_kwargs)
+            self.client = discovery.build_from_document(descriptor, **build_kwargs)
         else:
-            self.client = discovery.build('storage', 'v1', cache_discovery=False, **authenticate_kwargs)
+            build_kwargs.setdefault('cache_discovery', False)
+            self.client = discovery.build('storage', 'v1', **build_kwargs)
 
     def _path_to_bucket_and_key(self, path):
         (scheme, netloc, path, _, _) = urlsplit(path)
@@ -249,6 +253,35 @@ class GCSClient(luigi.target.FileSystem):
 
         self._do_put(media, dest_path)
 
+    def _forward_args_to_put(self, kwargs):
+        return self.put(**kwargs)
+
+    def put_multiple(self, filepaths, remote_directory, mimetype=None, chunksize=None, num_process=1):
+        if isinstance(filepaths, str):
+            raise ValueError(
+                'filenames must be a list of strings. If you want to put a single file, '
+                'use the `put(self, filename, ...)` method'
+            )
+
+        put_kwargs_list = [
+            {
+                'filename': filepath,
+                'dest_path': os.path.join(remote_directory, os.path.basename(filepath)),
+                'mimetype': mimetype,
+                'chunksize': chunksize,
+            }
+            for filepath in filepaths
+        ]
+
+        if num_process > 1:
+            from multiprocessing import Pool
+            from contextlib import closing
+            with closing(Pool(num_process)) as p:
+                return p.map(self._forward_args_to_put, put_kwargs_list)
+        else:
+            for put_kwargs in put_kwargs_list:
+                self._forward_args_to_put(put_kwargs)
+
     def put_string(self, contents, dest_path, mimetype=None):
         mimetype = mimetype or mimetypes.guess_type(dest_path)[0] or DEFAULT_MIMETYPE
         assert isinstance(mimetype, six.string_types)
