@@ -19,14 +19,16 @@ import logging
 
 import luigi
 
+from luigi.contrib import rdbms
+
 logger = logging.getLogger('luigi-interface')
 
 try:
     import mysql.connector
-    from mysql.connector import errorcode
+    from mysql.connector import errorcode, Error
 except ImportError as e:
     logger.warning("Loading MySQL module without the python package mysql-connector-python. \
-        This will crash at runtime if MySQL functionality is used.")
+       This will crash at runtime if MySQL functionality is used.")
 
 
 class MySqlTarget(luigi.Target):
@@ -147,3 +149,102 @@ class MySqlTarget(luigi.Target):
             else:
                 raise
         connection.close()
+
+
+class CopyToTable(rdbms.CopyToTable):
+    """
+    Template task for inserting a data set into MySQL
+
+    Usage:
+    Subclass and override the required `host`, `database`, `user`,
+    `password`, `table` and `columns` attributes.
+
+    To customize how to access data from an input task, override the `rows` method
+    with a generator that yields each row as a tuple with fields ordered according to `columns`.
+    """
+
+    def rows(self):
+        """
+        Return/yield tuples or lists corresponding to each row to be inserted.
+        """
+        with self.input().open('r') as fobj:
+            for line in fobj:
+                yield line.strip('\n').split('\t')
+
+# everything below will rarely have to be overridden
+
+    def output(self):
+        """
+        Returns a MySqlTarget representing the inserted dataset.
+
+        Normally you don't override this.
+        """
+        return MySqlTarget(
+            host=self.host,
+            database=self.database,
+            user=self.user,
+            password=self.password,
+            table=self.table,
+            update_id=self.update_id
+
+        )
+
+    def copy(self, cursor, file=None):
+        values = '({})'.format(','.join(['%s' for i in range(len(self.columns))]))
+        columns = '({})'.format(','.join([c[0] for c in self.columns]))
+        query = 'INSERT INTO {} {} VALUES {}'.format(self.table, columns, values)
+        rows = []
+
+        for idx, row in enumerate(self.rows()):
+            rows.append(row)
+
+            if (idx + 1) % self.bulk_size == 0:
+                cursor.executemany(query, rows)
+                rows = []
+
+        cursor.executemany(query, rows)
+
+    def run(self):
+        """
+        Inserts data generated by rows() into target table.
+
+        If the target table doesn't exist, self.create_table will be called to attempt to create the table.
+
+        Normally you don't want to override this.
+        """
+        if not (self.table and self.columns):
+            raise Exception("table and columns need to be specified")
+
+        connection = self.output().connect()
+
+        # attempt to copy the data into mysql
+        # if it fails because the target table doesn't exist
+        # try to create it by running self.create_table
+        for attempt in range(2):
+            try:
+                cursor = connection.cursor()
+                print("caling init copy...")
+                self.init_copy(connection)
+                self.copy(cursor)
+                self.post_copy(connection)
+                if self.enable_metadata_columns:
+                    self.post_copy_metacolumns(cursor)
+            except Error as err:
+                if err.errno == errorcode.ER_NO_SUCH_TABLE and attempt == 0:
+                    # if first attempt fails with "relation not found", try creating table
+                    # logger.info("Creating table %s", self.table)
+                    connection.reconnect()
+                    self.create_table(connection)
+                else:
+                    raise
+            else:
+                break
+
+        # mark as complete in same transaction
+        self.output().touch(connection)
+        connection.commit()
+        connection.close()
+
+    @property
+    def bulk_size(self):
+        return 10000
