@@ -22,7 +22,6 @@ See :doc:`/central_scheduler` for more info.
 """
 
 import collections
-import inspect
 import json
 
 from luigi.batch_notifier import BatchNotifier
@@ -97,7 +96,7 @@ def rpc_method(**request_args):
     def _rpc_method(fn):
         # If request args are passed, return this function again for use as
         # the decorator function with the request args attached.
-        fn_args = inspect.getargspec(fn)
+        fn_args = six.getargspec(fn)
 
         assert not fn_args.varargs
         assert fn_args.args[0] == 'self'
